@@ -207,24 +207,47 @@ module_id_dict = {'Campaigns': '1', 'Invoice': '2', 'SalesOrder': '3', 'Purchase
 def get_session_name(vtiger_access):
     vtiger_url = vtiger_access.get('vtiger_url')
     vtiger_username = vtiger_access.get('vtiger_username')
-    access_key_encoded = vtiger_access.get('vtiger_access_key')
+    user_access_key = vtiger_access.get('user_access_key')
 
-    if vtiger_url and vtiger_username and access_key_encoded:
+    if vtiger_url and vtiger_username and user_access_key:
 
-        login_url = "{vtiger_url}/webservice.php".format(vtiger_url=vtiger_url)
-        payload = 'operation=login&username={vtiger_username}&accessKey={access_key}'.format(
-            access_key=access_key_encoded, vtiger_username=vtiger_username)
-        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
-
-        response, status = invoke_http_request(login_url, 'POST', headers, payload)
+        # get challenge
+        headers = {'content-type': 'application/json'}
 
+        challenge_url = '{vtiger_url}/webservice.php?operation=getchallenge&username={vtiger_username}'.format(
+            vtiger_url=vtiger_url, vtiger_username=vtiger_username)
+        response, status = invoke_http_request(challenge_url, 'GET', headers=headers)
+        print("challenge url response, status", response, status)
         if status == 200 and response.get('result', ''):
-            print("existing session expired. New session id is: ", response.get('result').get('sessionName'))
-            return response.get('result').get('sessionName')
+            token = response.get('result').get('token')
+
+            # get md5 encoded access key
+
+            access_key = token + user_access_key
+            result = hashlib.md5(access_key.encode())
+            access_key_encoded = result.hexdigest()
+
+            # login
+            login_url = "{vtiger_url}/webservice.php".format(vtiger_url=vtiger_url)
+            payload = 'operation=login&username={vtiger_username}&accessKey={access_key}'.format(
+                access_key=access_key_encoded, vtiger_username=vtiger_username)
+            headers = {'Content-Type': 'application/x-www-form-urlencoded'}
+
+            response, status = invoke_http_request(login_url, 'POST', headers, payload)
+            print("login url response, status", response, status)
+
+            if response and status == 200 and response.get('result', ''):
+                print("existing session expired. New session id is: ", response.get('result').get('sessionName'))
+                return response.get('result').get('sessionName')
+            else:
+                print("Can not get Session name from Vtiger Login API. Error occurred with Status", status)
+                return None
         else:
-            print("Can not get Session name from Vtiger. Error occurred with Status", status)
+            print("Not able to get Challenge from API")
+            return None
     else:
         print("unable to get required data from vtiger_access")
+        return None
 
 
 def trigger_workflow(workflow, event_type, data, service_url, api_access_key, source_module):
@@ -376,8 +399,9 @@ def run_external_workflow(conf, task_id, vtiger_access):
                                     sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
                                 response, status = invoke_http_request(url, request_type, headers)
 
-                                vtiger_access['vtiger_session'] = session_name
-                                Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+                                if session_name:
+                                    vtiger_access['vtiger_session'] = session_name
+                                    Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
 
                 if response:
                     # trigger only if condition is satisfied
@@ -482,8 +506,9 @@ def trigger_set_value_task(set_value_fields, conf, rule, session_name, vtiger_ac
                         payload = {'operation': 'revise', 'sessionName': session_name, 'element': json.dumps(element)}
                         response, status = invoke_http_request(url, request_type, headers, payload=payload)
 
-                        vtiger_access['vtiger_session'] = session_name
-                        Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+                        if session_name:
+                            vtiger_access['vtiger_session'] = session_name
+                            Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
 
         print("Response: {}".format(response))
         if is_success_request(status):
@@ -594,8 +619,9 @@ def invoke_set_value_task(conf, task_id, vtiger_access, record_id=None):
                                 url = url.replace("''", "' '")
                                 response, status = invoke_http_request(url, request_type, headers)
 
-                                vtiger_access['vtiger_session'] = session_name
-                                Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+                                if session_name:
+                                    vtiger_access['vtiger_session'] = session_name
+                                    Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
 
                 print("Query Response: ")
 
@@ -842,8 +868,9 @@ def get_crm_records(vtiger_access, conf):
                             record_id=id, sessionName=session_name, vtiger_url=vtiger_access.get('vtiger_url'))
                         response, status = invoke_http_request(url, 'GET', {'content_type': 'application/json'})
 
-                        vtiger_access['vtiger_session'] = session_name
-                        Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+                        if session_name:
+                            vtiger_access['vtiger_session'] = session_name
+                            Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
 
         if 'success' in response and response['success'] and 'result' in response:
             return response['result']
@@ -914,8 +941,9 @@ def get_record_details_from_crm(vtiger_access, data):
                         response, status = invoke_http_request(url, 'GET', {'content_type': 'application/json'})
 
                         print("response, status", response, status)
-                        vtiger_access['vtiger_session'] = session_name
-                        Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+                        if session_name:
+                            vtiger_access['vtiger_session'] = session_name
+                            Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
 
         if 'success' in response and response['success'] and 'result' in response:
             # return list of user records
@@ -1056,8 +1084,9 @@ def run_external_workflow_with_fetch_configs(context, task_id, vtiger_access):
                                     sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
                                 response, status = invoke_http_request(url, request_type, headers)
 
-                                vtiger_access['vtiger_session'] = session_name
-                                Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+                                if session_name:
+                                    vtiger_access['vtiger_session'] = session_name
+                                    Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
 
                 if response:
                     # trigger only if condition is satisfied
